import { prop, ifProp, withProp, switchProp } from 'styled-tools';
import { a as numberToPx } from './chunk-84d8695a.js';
import React from 'react';
import PropTypes from 'prop-types';
import styled, { css } from 'styled-components';
import hoistNonReactStatics from 'hoist-non-react-statics';
import { a as as, b as classCallCheck, c as inherits, d as objectWithoutProperties, e as possibleConstructorReturn, f as _extends } from './as-8a1cd76d.js';
import { a as callAll } from './chunk-fa81e9f1.js';
import Base from './Base.js';
import { a as HiddenContainer } from './chunk-3cd03ec9.js';
import { a as HiddenHide, b as HiddenShow, c as HiddenToggle } from './chunk-60576777.js';
import 'lodash-es/uniq';
import 'react-known-props';
import 'lodash-es/omit';
import './chunk-a29c7f2d.js';
import 'lodash-es/kebabCase';
import 'constate';

var hasTransition = function hasTransition(props) {
  return Boolean(props.animated || props.fade || props.slide || props.expand);
};

var translate3d = function translate3d(x, y, z) {
  return "translate3d(" + numberToPx(x) + ", " + numberToPx(y) + ", " + numberToPx(z) + ")";
};

var origin = function origin() {
  var x = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "center";
  var y = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "center";
  return numberToPx(x) + " " + numberToPx(y);
};

var calc = function calc(a, b) {
  return "calc(" + numberToPx(a) + " + " + numberToPx(b) + ")";
};

var minus = function minus(v) {
  return "-" + numberToPx(v);
};

var expand = ifProp({ expand: true }, prop("defaultExpand", "center"), prop("expand"));

var slide = ifProp({ slide: true }, prop("defaultSlide", "right"), prop("slide"));

var scaleWithProps = ifProp("expand", "scale(0.01)");

var originWithProps = withProp(["originX", "originY"], function (x, y) {
  return switchProp(expand, {
    center: origin(calc("50%", x), calc("50%", y)),
    top: origin(calc("50%", x), calc("100%", y)),
    right: origin(x, calc("50%", y)),
    bottom: origin(calc("50%", x), y),
    left: origin(calc("100%", x), calc("50%", y))
  }, origin(calc("50%", x), calc("50%", y)));
});

var translateWithProps = withProp(["translateX", "translateY"], translate3d);

var slideWithProps = withProp(["translateX", "translateY", "slideOffset"], function (x, y) {
  var offset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "100%";
  return switchProp(slide, {
    top: translate3d(x, calc(offset, y)),
    right: translate3d(calc(minus(offset), x), y),
    bottom: translate3d(x, calc(minus(offset), y)),
    left: translate3d(calc(offset, x), y)
  }, translate3d(x, y));
});

var Component = function (_React$Component) {
  inherits(Component, _React$Component);

  function Component() {
    var _temp, _this, _ret;

    classCallCheck(this, Component);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
      visible: _this.props.visible,
      transitioning: _this.props.transitioning
    }, _this.applyState = function () {
      var _this$props = _this.props,
          visible = _this$props.visible,
          unmount = _this$props.unmount;


      if (typeof window !== "undefined" && unmount && hasTransition(_this.props)) {
        if (visible) {
          _this.setState({ transitioning: true });
          window.requestAnimationFrame(function () {
            return _this.setState({ transitioning: false, visible: true });
          });
        } else {
          _this.setState({ visible: false, transitioning: true });
        }
      } else {
        _this.setState({ visible: visible });
      }
    }, _this.handleTransitionEnd = function () {
      var _this$props2 = _this.props,
          visible = _this$props2.visible,
          unmount = _this$props2.unmount;

      if (!visible && unmount) {
        _this.setState({ transitioning: false });
      }
    }, _this.handleKeyDown = function (e) {
      var _this$props3 = _this.props,
          visible = _this$props3.visible,
          hide = _this$props3.hide;

      if (e.key === "Escape" && visible && hide) {
        hide();
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  Component.prototype.componentDidMount = function componentDidMount() {
    if (this.props.hideOnEsc) {
      document.body.addEventListener("keydown", this.handleKeyDown);
    }
  };

  Component.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (prevProps.visible !== this.props.visible) {
      this.applyState();
    }
  };

  Component.prototype.componentWillUnmount = function componentWillUnmount() {
    document.body.removeEventListener("keydown", this.handleKeyDown);
  };

  Component.prototype.render = function render() {
    var _props = this.props,
        unmount = _props.unmount,
        onTransitionEnd = _props.onTransitionEnd;
    var _state = this.state,
        visible = _state.visible,
        transitioning = _state.transitioning;


    if (unmount && !visible && !transitioning) {
      return null;
    }

    return React.createElement(Base, _extends({
      "aria-hidden": !visible
    }, this.props, this.state, {
      onTransitionEnd: callAll(this.handleTransitionEnd, onTransitionEnd)
    }));
  };

  return Component;
}(React.Component);

hoistNonReactStatics(Component, Base);

var Hidden = styled(Component).withConfig({
  displayName: "Hidden"
})(["transform:", ";", ";&[aria-hidden=\"true\"]{pointer-events:none;", ";", ";}", ";"], translateWithProps, ifProp(hasTransition, css(["transform-origin:", ";transition:all ", " ", " ", ";"], originWithProps, prop("duration"), prop("timing"), prop("delay"))), ifProp("fade", "opacity: 0"), ifProp(hasTransition, css(["transform:", " ", ";visibility:hidden;"], slideWithProps, scaleWithProps), "display: none !important"), prop("theme.Hidden"));

Hidden.propTypes = {
  visible: PropTypes.bool,
  hide: PropTypes.func,
  hideOnEsc: PropTypes.bool,
  unmount: PropTypes.bool,
  fade: PropTypes.bool,
  expand: PropTypes.oneOfType([PropTypes.bool, PropTypes.oneOf(["center", "top", "right", "bottom", "left"])]),
  slide: PropTypes.oneOfType([PropTypes.bool, PropTypes.oneOf(["top", "right", "bottom", "left"])]),
  duration: PropTypes.string,
  delay: PropTypes.string,
  timing: PropTypes.string,
  animated: PropTypes.bool
};

Hidden.defaultProps = {
  duration: "250ms",
  timing: "ease-in-out"
};

var Hidden$1 = as("div")(Hidden);

Hidden$1.Container = HiddenContainer;
Hidden$1.Hide = HiddenHide;
Hidden$1.Show = HiddenShow;
Hidden$1.Toggle = HiddenToggle;

export default Hidden$1;
