import React from 'react';
import { findDOMNode } from 'react-dom';
import PropTypes from 'prop-types';
import styled from 'styled-components';
import { prop } from 'styled-tools';
import Popper from 'popper.js';
import { a as as, b as classCallCheck, c as inherits, d as objectWithoutProperties, e as possibleConstructorReturn, f as _extends } from './as-8a1cd76d.js';
import Hidden from './Hidden.js';
import Box from './Box.js';
import { a as PopoverArrow } from './chunk-271f4c74.js';
import uniqueId from 'lodash-es/uniqueId';
import { a as HiddenContainer } from './chunk-3cd03ec9.js';
import { a as callAll } from './chunk-fa81e9f1.js';
import { c as HiddenToggle, b as HiddenShow, a as HiddenHide } from './chunk-60576777.js';
import 'lodash-es/uniq';
import 'react-known-props';
import 'lodash-es/omit';
import './chunk-84d8695a.js';
import 'hoist-non-react-statics';
import './Base.js';
import './chunk-a29c7f2d.js';
import 'lodash-es/kebabCase';
import 'constate';
import './Arrow.js';

/* eslint-disable react/no-find-dom-node */

var HiddenBox = Hidden.as(Box);

var Component = function (_React$Component) {
  inherits(Component, _React$Component);

  function Component() {
    var _temp, _this, _ret;

    classCallCheck(this, Component);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
      placement: _this.props.placement,
      translateX: undefined,
      translateY: undefined,
      originX: undefined,
      originY: undefined
    }, _this.getPopover = function () {
      return findDOMNode(_this);
    }, _this.modifier = function (data) {
      var placement = data.placement,
          offsets = data.offsets,
          arrowElement = data.arrowElement,
          arrowStyles = data.arrowStyles;
      var reference = offsets.reference,
          popper = offsets.popper;

      var _placement$split = placement.split("-"),
          position = _placement$split[0];

      var isVertical = ["top", "bottom"].indexOf(position) >= 0;

      var referenceCenter = isVertical ? reference.width / 2 : reference.height / 2;
      var side = isVertical ? "left" : "top";
      var sideValue = referenceCenter - popper[side];

      if (arrowElement) {
        var top = arrowStyles.top,
            left = arrowStyles.left;

        arrowElement.style.top = isVertical ? "" : top + "px";
        arrowElement.style.left = isVertical ? left + "px" : "";
      }

      _this.setState({
        originX: isVertical ? sideValue + "px - 50%" : 0,
        originY: !isVertical ? sideValue + "px - 50%" : 0,
        translateX: popper.left,
        translateY: popper.top,
        placement: data.placement
      });

      return data;
    }, _this.getOptions = function () {
      var placement = _this.props.placement;

      var popover = _this.getPopover();
      var arrowClassName = "." + PopoverArrow.styledComponentId;
      var arrow = popover.querySelector(arrowClassName);
      return {
        placement: placement,
        modifiers: {
          applyStyle: { enabled: false },
          arrow: { enabled: !!arrow, element: arrow },
          flip: { padding: 16 },
          offset: { offset: "0, " + _this.props.gutter },
          setState: {
            enabled: true,
            order: 900,
            fn: _this.modifier
          }
        }
      };
    }, _this.initPopper = function () {
      if (!_this.popper) {
        var popover = _this.getPopover();
        _this.popper = new Popper(popover.parentNode, popover, _this.getOptions());
      }
    }, _this.destroyPopper = function () {
      if (_this.popper) {
        _this.popper.destroy();
        _this.popper = undefined;
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  Component.prototype.componentDidMount = function componentDidMount() {
    this.initPopper();
  };

  Component.prototype.componentWillUnmount = function componentWillUnmount() {
    this.destroyPopper();
  };

  Component.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (prevProps.visible !== this.props.visible || prevProps.placement !== this.props.placement || prevProps.children !== this.props.children) {
      this.destroyPopper();
      this.initPopper();
    }
  };

  Component.prototype.render = function render() {
    return React.createElement(HiddenBox, _extends({
      id: this.props.popoverId
    }, this.state, {
      "data-placement": this.state.placement,
      defaultSlide: this.state.placement.replace(/-.+$/, ""),
      defaultExpand: this.state.placement.replace(/-.+$/, ""),
      slideOffset: this.props.gutter
    }, this.props));
  };

  return Component;
}(React.Component);

var Popover = styled(Component).withConfig({
  displayName: "Popover"
})(["position:absolute;top:0;left:0;user-select:auto;cursor:auto;color:inherit;background-color:white;padding:1em;z-index:999;outline:0;will-change:transform;&[aria-hidden=\"false\"]{transition-timing-function:", ";}", ";"], prop("timing", "cubic-bezier(0.25, 0.1, 0.25, 1.5)"), prop("theme.Popover"));

Popover.propTypes = {
  placement: PropTypes.oneOf(["auto", "top", "right", "bottom", "left", "top-start", "right-start", "bottom-start", "left-start", "top-end", "right-end", "bottom-end", "left-end"]),
  gutter: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  popoverId: PropTypes.string
};

Popover.defaultProps = {
  role: "group",
  placement: "bottom",
  hideOnEsc: true,
  gutter: 12
};

var Popover$1 = as("div")(Popover);

var onMount = function onMount(_ref) {
  var setState = _ref.setState;

  setState({ popoverId: uniqueId("popover") });
};

var PopoverContainer = function PopoverContainer(props) {
  return React.createElement(HiddenContainer, _extends({}, props, { onMount: callAll(onMount, props.onMount) }));
};

var Component$1 = function Component(props) {
  return React.createElement(HiddenToggle, _extends({
    "aria-expanded": props.visible,
    "aria-controls": props.popoverId,
    "aria-haspopup": true
  }, props));
};

var PopoverToggle = styled(Component$1).withConfig({
  displayName: "PopoverToggle"
})(["", ";"], prop("theme.PopoverToggle"));

PopoverToggle.propTypes = {
  popoverId: PropTypes.string,
  visible: PropTypes.bool
};

var PopoverToggle$1 = as("button")(PopoverToggle);

var Component$2 = function Component(props) {
  return React.createElement(HiddenShow, _extends({
    "aria-expanded": props.visible,
    "aria-controls": props.popoverId,
    "aria-haspopup": true
  }, props));
};

var PopoverShow = styled(Component$2).withConfig({
  displayName: "PopoverShow"
})(["", ";"], prop("theme.PopoverShow"));

PopoverShow.propTypes = {
  popoverId: PropTypes.string,
  visible: PropTypes.bool
};

var PopoverShow$1 = as("button")(PopoverShow);

var Component$3 = function Component(props) {
  return React.createElement(HiddenHide, _extends({
    "aria-expanded": props.visible,
    "aria-controls": props.popoverId,
    "aria-haspopup": true
  }, props));
};

var PopoverHide = styled(Component$3).withConfig({
  displayName: "PopoverHide"
})(["", ";"], prop("theme.PopoverHide"));

PopoverHide.propTypes = {
  popoverId: PropTypes.string,
  visible: PropTypes.bool
};

var PopoverHide$1 = as("button")(PopoverHide);

Popover$1.Arrow = PopoverArrow;
Popover$1.Container = PopoverContainer;
Popover$1.Toggle = PopoverToggle$1;
Popover$1.Show = PopoverShow$1;
Popover$1.Hide = PopoverHide$1;

export default Popover$1;
