import styled from 'styled-components';
import { prop } from 'styled-tools';
import { a as as, b as classCallCheck, c as inherits, d as objectWithoutProperties, e as possibleConstructorReturn, f as _extends } from './as-8a1cd76d.js';
import Base from './Base.js';
import React from 'react';
import PropTypes from 'prop-types';
import { a as StepContainer, b as StepNext, c as StepPrevious } from './chunk-6fa58b5b.js';
import { a as callAll } from './chunk-fa81e9f1.js';
import Step from './Step.js';
import hoistNonReactStatics from 'hoist-non-react-statics';
import Hidden from './Hidden.js';
import 'lodash-es/uniq';
import 'react-known-props';
import 'lodash-es/omit';
import './chunk-a29c7f2d.js';
import 'lodash-es/kebabCase';
import './chunk-84d8695a.js';
import 'constate';
import './chunk-60576777.js';
import './chunk-3cd03ec9.js';

var Tabs = styled(Base).withConfig({
  displayName: "Tabs"
})(["display:flex;align-items:center;list-style:none;@media screen and (max-width:640px){overflow-x:auto;}", ";"], prop("theme.Tabs"));

Tabs.defaultProps = {
  role: "tablist"
};

var Tabs$1 = as("ul")(Tabs);

var initialState = {
  loop: true,
  current: 0
};

// istanbul ignore next
var TabsContainer = function TabsContainer(props) {
  return React.createElement(StepContainer, _extends({}, props, {
    initialState: _extends({}, initialState, props.initialState)
  }));
};

TabsContainer.propTypes = {
  initialState: PropTypes.object
};

/* eslint-disable no-param-reassign */
var createElementRef = function createElementRef(scope, property) {
  return function (element) {
    scope[property] = element;
    if (scope.props && scope.props.elementRef) {
      scope.props.elementRef(element);
    }
  };
};

var Component = function (_React$Component) {
  inherits(Component, _React$Component);

  function Component() {
    var _temp, _this, _ret;

    classCallCheck(this, Component);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.show = function () {
      var _this$props = _this.props,
          show = _this$props.show,
          isCurrent = _this$props.isCurrent,
          tab = _this$props.tab;

      if (!isCurrent(tab)) {
        show(tab);
      }
    }, _this.keyDown = function (e) {
      var keyMap = {
        ArrowLeft: _this.props.previous,
        ArrowRight: _this.props.next
      };
      if (keyMap[e.key]) {
        e.preventDefault();
        keyMap[e.key]();
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  Component.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    var _props = this.props,
        current = _props.current,
        isCurrent = _props.isCurrent,
        tab = _props.tab;


    if (prevProps.current !== current && isCurrent(tab)) {
      this.element.focus();
    }
  };

  Component.prototype.render = function render() {
    var _props2 = this.props,
        isCurrent = _props2.isCurrent,
        tab = _props2.tab,
        className = _props2.className,
        onClick = _props2.onClick,
        onFocus = _props2.onFocus,
        onKeyDown = _props2.onKeyDown;


    var active = isCurrent(tab);
    var activeClassName = active ? "active" : "";
    var finalClassName = [className, activeClassName].filter(Boolean).join(" ");

    return React.createElement(Step, _extends({
      id: tab + "Tab",
      step: tab,
      active: active,
      "aria-selected": active,
      "aria-controls": tab + "Panel",
      tabIndex: active ? 0 : -1,
      visible: true
    }, this.props, {
      onClick: callAll(this.show, onClick),
      onFocus: callAll(this.show, onFocus),
      onKeyDown: callAll(this.keyDown, onKeyDown),
      className: finalClassName,
      elementRef: createElementRef(this, "element")
    }));
  };

  return Component;
}(React.Component);

var TabsTab = styled(Component).withConfig({
  displayName: "TabsTab"
})(["display:inline-flex;position:relative;flex:1;user-select:none;outline:none;align-items:center;white-space:nowrap;justify-content:center;text-decoration:none;height:2.5em;padding:0 0.5em;min-width:2.5em;&.active{font-weight:bold;}&[disabled]{pointer-events:none;}", ";"], prop("theme.TabsTab"));

TabsTab.propTypes = {
  tab: PropTypes.string.isRequired,
  register: PropTypes.func.isRequired,
  update: PropTypes.func.isRequired,
  unregister: PropTypes.func.isRequired,
  isCurrent: PropTypes.func.isRequired,
  show: PropTypes.func.isRequired,
  next: PropTypes.func.isRequired,
  previous: PropTypes.func.isRequired,
  className: PropTypes.string,
  disabled: PropTypes.bool,
  onClick: PropTypes.func,
  onFocus: PropTypes.func,
  onKeyDown: PropTypes.func
};

var noop = function noop() {};

TabsTab.defaultProps = {
  role: "tab",
  register: noop,
  update: noop,
  unregister: noop,
  isCurrent: noop,
  show: noop,
  next: noop,
  previous: noop
};

var TabsTab$1 = as("li")(TabsTab);

var Component$1 = function Component(props) {
  var isCurrent = props.isCurrent,
      tab = props.tab;

  return React.createElement(Hidden, _extends({
    id: tab + "Panel",
    "aria-labelledby": tab + "Tab",
    unmount: true,
    visible: isCurrent && isCurrent(tab)
  }, props));
};

hoistNonReactStatics(Component$1, Hidden);

var TabsPanel = styled(Component$1).withConfig({
  displayName: "TabsPanel"
})(["", ";"], prop("theme.TabsPanel"));

TabsPanel.propTypes = {
  tab: PropTypes.string.isRequired,
  isCurrent: PropTypes.func.isRequired
};

TabsPanel.defaultProps = {
  role: "tabpanel"
};

var TabsPanel$1 = as("div")(TabsPanel);

var TabsNext = styled(StepNext).withConfig({
  displayName: "TabsNext"
})(["", ";"], prop("theme.TabsNext"));

var TabsNext$1 = as("button")(TabsNext);

var TabsPrevious = styled(StepPrevious).withConfig({
  displayName: "TabsPrevious"
})(["", ";"], prop("theme.TabsPrevious"));

var TabsPrevious$1 = as("button")(TabsPrevious);

Tabs$1.Container = TabsContainer;
Tabs$1.Tab = TabsTab$1;
Tabs$1.Panel = TabsPanel$1;
Tabs$1.Next = TabsNext$1;
Tabs$1.Previous = TabsPrevious$1;

export default Tabs$1;
