'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var styledTools = require('styled-tools');
var __chunk_1 = require('./chunk-c628ddbd.js');
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var styled = require('styled-components');
var styled__default = _interopDefault(styled);
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));
var as = require('./as-abd2c25b.js');
var __chunk_3 = require('./chunk-c7f60f29.js');
var Base = require('./Base.js');
var __chunk_4 = require('./chunk-df525803.js');
var __chunk_5 = require('./chunk-e0ed6a07.js');
require('lodash/uniq');
require('react-known-props');
require('lodash/omit');
require('./chunk-7e606c56.js');
require('lodash/kebabCase');
require('constate');

var hasTransition = function hasTransition(props) {
  return Boolean(props.animated || props.fade || props.slide || props.expand);
};

var translate3d = function translate3d(x, y, z) {
  return "translate3d(" + __chunk_1.numberToPx(x) + ", " + __chunk_1.numberToPx(y) + ", " + __chunk_1.numberToPx(z) + ")";
};

var origin = function origin() {
  var x = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "center";
  var y = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "center";
  return __chunk_1.numberToPx(x) + " " + __chunk_1.numberToPx(y);
};

var calc = function calc(a, b) {
  return "calc(" + __chunk_1.numberToPx(a) + " + " + __chunk_1.numberToPx(b) + ")";
};

var minus = function minus(v) {
  return "-" + __chunk_1.numberToPx(v);
};

var expand = styledTools.ifProp({ expand: true }, styledTools.prop("defaultExpand", "center"), styledTools.prop("expand"));

var slide = styledTools.ifProp({ slide: true }, styledTools.prop("defaultSlide", "right"), styledTools.prop("slide"));

var scaleWithProps = styledTools.ifProp("expand", "scale(0.01)");

var originWithProps = styledTools.withProp(["originX", "originY"], function (x, y) {
  return styledTools.switchProp(expand, {
    center: origin(calc("50%", x), calc("50%", y)),
    top: origin(calc("50%", x), calc("100%", y)),
    right: origin(x, calc("50%", y)),
    bottom: origin(calc("50%", x), y),
    left: origin(calc("100%", x), calc("50%", y))
  }, origin(calc("50%", x), calc("50%", y)));
});

var translateWithProps = styledTools.withProp(["translateX", "translateY"], translate3d);

var slideWithProps = styledTools.withProp(["translateX", "translateY", "slideOffset"], function (x, y) {
  var offset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "100%";
  return styledTools.switchProp(slide, {
    top: translate3d(x, calc(offset, y)),
    right: translate3d(calc(minus(offset), x), y),
    bottom: translate3d(x, calc(minus(offset), y)),
    left: translate3d(calc(offset, x), y)
  }, translate3d(x, y));
});

var Component = function (_React$Component) {
  as.inherits(Component, _React$Component);

  function Component() {
    var _temp, _this, _ret;

    as.classCallCheck(this, Component);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = as.possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.state = {
      visible: _this.props.visible,
      transitioning: _this.props.transitioning
    }, _this.applyState = function () {
      var _this$props = _this.props,
          visible = _this$props.visible,
          unmount = _this$props.unmount;


      if (typeof window !== "undefined" && unmount && hasTransition(_this.props)) {
        if (visible) {
          _this.setState({ transitioning: true });
          window.requestAnimationFrame(function () {
            return _this.setState({ transitioning: false, visible: true });
          });
        } else {
          _this.setState({ visible: false, transitioning: true });
        }
      } else {
        _this.setState({ visible: visible });
      }
    }, _this.handleTransitionEnd = function () {
      var _this$props2 = _this.props,
          visible = _this$props2.visible,
          unmount = _this$props2.unmount;

      if (!visible && unmount) {
        _this.setState({ transitioning: false });
      }
    }, _this.handleKeyDown = function (e) {
      var _this$props3 = _this.props,
          visible = _this$props3.visible,
          hide = _this$props3.hide;

      if (e.key === "Escape" && visible && hide) {
        hide();
      }
    }, _temp), as.possibleConstructorReturn(_this, _ret);
  }

  Component.prototype.componentDidMount = function componentDidMount() {
    if (this.props.hideOnEsc) {
      document.body.addEventListener("keydown", this.handleKeyDown);
    }
  };

  Component.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (prevProps.visible !== this.props.visible) {
      this.applyState();
    }
  };

  Component.prototype.componentWillUnmount = function componentWillUnmount() {
    document.body.removeEventListener("keydown", this.handleKeyDown);
  };

  Component.prototype.render = function render() {
    var _props = this.props,
        unmount = _props.unmount,
        onTransitionEnd = _props.onTransitionEnd;
    var _state = this.state,
        visible = _state.visible,
        transitioning = _state.transitioning;


    if (unmount && !visible && !transitioning) {
      return null;
    }

    return React.createElement(Base.default, as._extends({
      "aria-hidden": !visible
    }, this.props, this.state, {
      onTransitionEnd: __chunk_3.callAll(this.handleTransitionEnd, onTransitionEnd)
    }));
  };

  return Component;
}(React.Component);

hoistNonReactStatics(Component, Base.default);

var Hidden = styled__default(Component).withConfig({
  displayName: "Hidden"
})(["transform:", ";", ";&[aria-hidden=\"true\"]{pointer-events:none;", ";", ";}", ";"], translateWithProps, styledTools.ifProp(hasTransition, styled.css(["transform-origin:", ";transition:all ", " ", " ", ";"], originWithProps, styledTools.prop("duration"), styledTools.prop("timing"), styledTools.prop("delay"))), styledTools.ifProp("fade", "opacity: 0"), styledTools.ifProp(hasTransition, styled.css(["transform:", " ", ";visibility:hidden;"], slideWithProps, scaleWithProps), "display: none !important"), styledTools.prop("theme.Hidden"));

Hidden.propTypes = {
  visible: PropTypes.bool,
  hide: PropTypes.func,
  hideOnEsc: PropTypes.bool,
  unmount: PropTypes.bool,
  fade: PropTypes.bool,
  expand: PropTypes.oneOfType([PropTypes.bool, PropTypes.oneOf(["center", "top", "right", "bottom", "left"])]),
  slide: PropTypes.oneOfType([PropTypes.bool, PropTypes.oneOf(["top", "right", "bottom", "left"])]),
  duration: PropTypes.string,
  delay: PropTypes.string,
  timing: PropTypes.string,
  animated: PropTypes.bool
};

Hidden.defaultProps = {
  duration: "250ms",
  timing: "ease-in-out"
};

var Hidden$1 = as.as("div")(Hidden);

Hidden$1.Container = __chunk_4.HiddenContainer;
Hidden$1.Hide = __chunk_5.HiddenHide;
Hidden$1.Show = __chunk_5.HiddenShow;
Hidden$1.Toggle = __chunk_5.HiddenToggle;

exports.default = Hidden$1;
