'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var omit = _interopDefault(require('lodash/omit'));
var constate = require('constate');
var as = require('./as-abd2c25b.js');
var styled = require('styled-components');
var styled__default = _interopDefault(styled);
var styledTools = require('styled-tools');
var Base = require('./Base.js');
var __chunk_3 = require('./chunk-c7f60f29.js');

/* eslint-disable react/no-unused-prop-types */

var initialState = {
  loop: false,
  ids: [],
  current: -1,
  ordered: {}
};

var getCurrentId = function getCurrentId() {
  return function (state) {
    return state.ids[state.current];
  };
};

var hasPrevious = function hasPrevious() {
  return function (state) {
    return state.ids.length > 1 && !!state.ids[state.current - 1];
  };
};

var hasNext = function hasNext() {
  return function (state) {
    return state.ids.length > 1 && !!state.ids[state.current + 1];
  };
};

var indexOf = function indexOf(idOrIndex) {
  return function (state) {
    return typeof idOrIndex === "number" ? idOrIndex : state.ids.indexOf(idOrIndex);
  };
};

var isCurrent = function isCurrent(idOrIndex) {
  return function (state) {
    return state.current >= 0 && state.current === indexOf(idOrIndex)(state);
  };
};

var show = function show(idOrIndex) {
  return function (state) {
    return {
      current: indexOf(idOrIndex)(state)
    };
  };
};

var hide = function hide() {
  return function () {
    return { current: -1 };
  };
};

var toggle = function toggle(idOrIndex) {
  return function (state) {
    return isCurrent(idOrIndex)(state) ? hide()(state) : show(idOrIndex)(state);
  };
};

var previous = function previous() {
  return function (state) {
    if (hasPrevious()(state)) {
      return { current: state.current - 1 };
    }
    if (state.loop) {
      return { current: state.ids.length - 1 };
    }
    return {};
  };
};

var next = function next() {
  return function (state) {
    if (hasNext()(state)) {
      return { current: state.current + 1 };
    }
    if (state.loop) {
      return { current: 0 };
    }
    return {};
  };
};

var reorder = function reorder(id) {
  var order = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  return function (state) {
    var _babelHelpers$extends;

    var ordered = as._extends({}, state.ordered, (_babelHelpers$extends = {}, _babelHelpers$extends[id] = order, _babelHelpers$extends));
    var ids = state.ids.slice().sort(function (a, b) {
      return (ordered[a] || 0) - (ordered[b] || 0);
    });
    return as._extends({
      ordered: ordered,
      ids: ids
    }, isCurrent(id)(state) ? show(id)(as._extends({}, state, { ids: ids })) : {});
  };
};

var register = function register(id) {
  var order = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  return function (state) {
    var ids = state.ids.indexOf(id) >= 0 ? state.ids : [].concat(state.ids, [id]);
    return reorder(id, order)(as._extends({}, state, { ids: ids }));
  };
};

var unregister = function unregister(id) {
  return function (state) {
    var index = indexOf(id)(state);
    if (index === -1) {
      return {};
    }

    var ordered = omit(state.ordered, state.ids[index]);
    var ids = [].concat(state.ids.slice(0, index), state.ids.slice(index + 1));

    if (isCurrent(id)(state) && !hasNext()(state)) {
      if (hasPrevious()(state)) {
        return as._extends({}, previous()(state), { ids: ids, ordered: ordered });
      }
      return as._extends({}, hide()(state), { ids: ids, ordered: ordered });
    }
    if (state.current >= ids.length) {
      return { current: ids.length - 1, ids: ids, ordered: ordered };
    }
    return { ids: ids, ordered: ordered };
  };
};

var update = function update(id, nextId, orderArg) {
  return function (state) {
    var order = typeof orderArg !== "undefined" ? orderArg : state.ordered[id];
    var idChanged = id !== nextId;
    var orderChanged = order !== state.ordered[id];

    if (!idChanged && !orderChanged) return {};

    var overridingId = idChanged && state.ids.indexOf(nextId) >= 0;

    if (overridingId) {
      var nextOrderChanged = order !== state.ordered[nextId];
      var nextState = nextOrderChanged ? reorder(nextId, order)(state) : {};
      return unregister(id)(as._extends({}, state, nextState));
    }

    var index = indexOf(id)(state);
    var ids = [].concat(state.ids.slice(0, index), [nextId], state.ids.slice(index + 1));
    return reorder(nextId, order)(as._extends({}, state, { ids: ids }));
  };
};

var selectors = {
  getCurrentId: getCurrentId,
  hasPrevious: hasPrevious,
  hasNext: hasNext,
  indexOf: indexOf,
  isCurrent: isCurrent
};

var actions = {
  show: show,
  hide: hide,
  toggle: toggle,
  previous: previous,
  next: next,
  reorder: reorder,
  register: register,
  unregister: unregister,
  update: update
};

// istanbul ignore next
var StepContainer = function StepContainer(props) {
  return React.createElement(constate.Container, as._extends({}, props, {
    initialState: as._extends({}, initialState, props.initialState),
    selectors: as._extends({}, selectors, props.selectors),
    actions: as._extends({}, actions, props.actions)
  }));
};

StepContainer.propTypes = {
  initialState: PropTypes.object,
  actions: PropTypes.objectOf(PropTypes.func),
  selectors: PropTypes.objectOf(PropTypes.func)
};

var Component = function Component(_ref) {
  var onClick = _ref.onClick,
      props = as.objectWithoutProperties(_ref, ["onClick"]);
  return React.createElement(Base.default, as._extends({
    onClick: __chunk_3.callAll(props.next, onClick),
    disabled: !props.loop && props.hasNext && !props.hasNext()
  }, props));
};

var StepNext = styled__default(Component).withConfig({
  displayName: "StepNext"
})(["", ";"], styledTools.prop("theme.StepNext"));

StepNext.propTypes = {
  next: PropTypes.func.isRequired,
  hasNext: PropTypes.func,
  loop: PropTypes.bool,
  onClick: PropTypes.func
};

var StepNext$1 = as.as("button")(StepNext);

var Component$1 = function Component(_ref) {
  var onClick = _ref.onClick,
      props = as.objectWithoutProperties(_ref, ["onClick"]);
  return React.createElement(Base.default, as._extends({
    onClick: __chunk_3.callAll(props.previous, onClick),
    disabled: !props.loop && props.hasPrevious && !props.hasPrevious()
  }, props));
};

var StepPrevious = styled__default(Component$1).withConfig({
  displayName: "StepPrevious"
})(["", ";"], styledTools.prop("theme.StepPrevious"));

StepPrevious.propTypes = {
  previous: PropTypes.func.isRequired,
  hasPrevious: PropTypes.func,
  loop: PropTypes.bool,
  onClick: PropTypes.func
};

var StepPrevious$1 = as.as("button")(StepPrevious);

exports.StepContainer = StepContainer;
exports.StepNext = StepNext$1;
exports.StepPrevious = StepPrevious$1;
